
import java.io.FileReader;
import java.util.*;

class TestLexiconLookup {

    public static void main(String[] args) {
        try {
            readAndTestLexicon(args);
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println();
            System.err.println("Usage: java TestLexiconLookup <class-to-test> <textfile> <lexicon-size> <nr-tests>");
            System.err.println("  where <class-to-test> is: HashSet, TreeSet, UnorderedArraySet, BinarySearchTree");
            System.err.println();
            System.exit(1);
        }
    }
    
    public static void readAndTestLexicon(String[] args) throws Exception {
        if (args.length != 4) 
            throw new IllegalArgumentException();
        String classToTest = args[0];
        String textfile = args[1];
        int nrWords = Integer.parseInt(args[2]);
        int nrTests = Integer.parseInt(args[3]);

        System.out.format("Creating %s lexicon from textfile: %s\n", classToTest, textfile);
        Set<String> lexicon = createLexicon(classToTest);
        double creationTime = readLexicon(lexicon, textfile, nrWords);
        System.out.format("Time to build lexicon: %.3f ms\n", creationTime/1e6);
        System.out.println();

        System.out.format("      Nr. tested    Total time      Time per word     Lexicon size\n");
        for (int i = 0; i < nrTests; i++) {
            double testingTime = testLexicon(lexicon, textfile, nrWords);
            System.out.format("%10d words %10.3f ms %10.3f us/word %10d words\n", 
                              nrWords, testingTime/1e6, testingTime/1e3/nrWords, lexicon.size());
        }
        System.out.println();
    }

    private static double readLexicon(Set<String> lexicon, String textfile, int lexiconSize) throws Exception {
        long creationTime = 0;
        Scanner scanner = new Scanner(new FileReader(textfile));
        while (lexicon.size() < lexiconSize) {
            if (!scanner.hasNext()) throw new IndexOutOfBoundsException("Too few words in file!");
            String word = scanner.next();
            creationTime -= System.nanoTime();
            lexicon.add(word);
            creationTime += System.nanoTime();
        }
        scanner.close();
        return (double) creationTime;
    }

    private static double testLexicon(Set<String> lexicon, String textfile, int nrWords) throws Exception {
        long testingTime = 0;
        Scanner scanner = new Scanner(new FileReader(textfile));
        for (int i = 0; i < nrWords; i++) {
            String word = scanner.next();
            testingTime -= System.nanoTime();
            boolean contained = lexicon.contains(word);
            testingTime += System.nanoTime();
            assert contained;
        }
        scanner.close();
        return (double) testingTime;
    }

    private static Set<String> createLexicon(String className) throws IllegalArgumentException {
        if (className.equals("HashSet"))           return new HashSet<String>();
        if (className.equals("TreeSet"))           return new TreeSet<String>();
        if (className.equals("UnorderedArraySet")) return new UnorderedArraySet<String>();
        if (className.equals("BinarySearchTree"))  return new BinarySearchTree<String>();
        throw new IllegalArgumentException();
    }

}

